// Copyright 2019 utimes.cc. All rights reserved.
// Use of this source code is governed by utimes.cc.

package data_api_check

import (
	"context"
	"gd_auth_check/apis"
	"time"

	//	"gd_auth_check/common.in/utils"
	"gd_auth_check/errors"
	"gd_auth_check/rpc_apis"
	"gd_auth_check/rpc_apis/gd_access_log"
	"fmt"
	"strings"

	"github.com/astaxie/beego/orm"
)

func addAccessFailedCount(o orm.Ormer, accessCountId int64) error {
	sql := "update t_gd_api_access_count set fail_count=fail_count+1 where id=?"
	_, err := o.Raw(sql, accessCountId).Exec()
	if err != nil {
		setMysqlError(sql, fmt.Sprintf("%s", accessCountId), err)
		return errors.DataBaseError
	}
	return nil
}

func decreaseAccessCount(o orm.Ormer, accessCountId int64) error {
	count := 0
	sql := "select count from t_gd_api_access_count where id=?"
	err := o.Raw(sql, accessCountId).QueryRow(&count)
	if err != nil {
		setMysqlError(sql, fmt.Sprintf("%s", accessCountId), err)
		return errors.DataBaseError
	}
	if count == 0 {
		return nil
	}
	sql = "update t_gd_api_access_count set count= case when count > 0 then count-1 else 0 end, day_count= case when day_count > 0 and create_time='0000-00-00' then day_count - 1 else day_count end where id=?"
	_, err = o.Raw(sql, accessCountId).Exec()
	if err != nil {
		setMysqlError(sql, fmt.Sprintf("%s", accessCountId), err)
		return errors.DataBaseError
	}
	return nil
}

func apiWarning(id int64) {
	req := gd_access_log.WarningApiCountReq{}

	mdaid := int64(0)
	usedCount := 0
	countAfterIncrease := 0
	usedBeforeIncrease := 0
	o := orm.NewOrm()

	sql := "select merchant_data_api_id, count from t_gd_api_access_count where id = ?"
	err := o.Raw(sql, id).QueryRow(&mdaid, &usedCount)
	if err != nil {
		return
	}
	req.MerchantDataApiId = mdaid
	enable := false
	sql = "select b.email, name as user, warning_enable from (t_gd_user as a inner join t_gd_merchants as b on a.id=b.user_id) inner join t_gd_merchant_data_api as c on c.merchant_id=b.id where c.id=?"
	err = o.Raw(sql, mdaid).QueryRow(&req.Email, &req.User, &enable)
	if err != nil {
		return
	}
	countThresholdsEnable := 0
	sql = `SELECT
			(select data_api_name from t_gd_data_api where id=a.data_api_id) as api_name,
			a.is_free,
			b.count AS total_count,
			b.count_per_day AS day_count,
			b.combo_type,
			b.end_time, 
			b.count_after_increase,
			b.used_before_increase,
			b.count_thresholds,
			b.day_thresholds,
			b.count_thresholds_enable
			FROM
			t_gd_merchant_data_api AS b
			LEFT JOIN t_gd_data_api_query_type AS a ON a.id = b.query_type_id
			WHERE
			b.id = ?`
	err = o.Raw(sql, mdaid).QueryRow(&req.ApiName,
		&req.IsFree,
		&req.TotalCount,
		&req.DayCount,
		&req.ComboType, &req.EndTime,
		&countAfterIncrease, &usedBeforeIncrease,
		&req.CountThresholds, &req.DayThresholds, &countThresholdsEnable)
	if err != nil {
		return
	}
	if enable == false {
		if req.CountThresholds == "" && req.ComboType == 1 {
			return
		}
		if req.CountThresholds == "" && req.DayThresholds == "" && req.ComboType == 2 {
			return
		}
	}
	if countThresholdsEnable == 0 {
		req.CountThresholds = ""
		req.DayThresholds = ""
	}
	req.Enable = enable
	if req.ComboType == 1 {
		req.RealUsed = usedCount
		req.RealTotal = req.TotalCount
		req.TotalUsed = usedCount
		if countAfterIncrease > 0 && usedCount-usedBeforeIncrease >= 0 {
			req.TotalCount = countAfterIncrease
			req.TotalUsed = usedCount - usedBeforeIncrease
		}
	} else {
		req.DayUsed = usedCount
	}
	rpc_apis.Crontab.WarningApiCount(context.Background(), &req)
}

// 金额扣费方式
func MoneyChargeMode(ctx context.Context, req *apis.ManagementCheckCountCodeReq, reply *apis.ManagementCheckCountCodeReply) (err error) {
	// 插入t_gd_consume (存在更新，不存在插入)
	// insert into t_gd_consume(day,month,merchant_data_api_id,merchant_id,total_count,valid_count,charge_count,amount) value ("2022-01","2022-01-21",1,2,1,1,1,1.11) ON DUPLICATE key UPDATE valid_count=valid_count+1
	if req.MerchantId == 0 || req.MerchantDataApiId == 0 {
		return nil
	}
	timeNow := time.Now()
	day := timeNow.Format("2006-01-02")
	month := timeNow.Format("2006-01")
	sql := "insert into t_gd_consume(day,month,merchant_data_api_id,merchant_id,total_count,valid_count,charge_count,amount,remark) " +
		"value ('%s','%s',%d,%d,1,%d,%d,%f,'') ON DUPLICATE key UPDATE total_count=total_count+1,valid_count=valid_count+%d," +
		"charge_count=charge_count+%d,amount=amount+%f"
	validCount := 0
	chargeCount := 0
	amount := float64(0)
	if req.Code == "0" || req.Code == "1100" {
		validCount = 1
	}

	if req.CountType == 0 {
		chargeCount = 1
		amount = req.UnitPrice
	} else {
		array := strings.Split(req.CountCode, ",")
		for _, v := range array {
			if v == req.Code {
				chargeCount = 1
				amount = req.UnitPrice
				break
			}
		}
	}

	o := orm.NewOrm()
	o.Begin()
	sql = fmt.Sprintf(sql, day, month, req.MerchantDataApiId, req.MerchantId, validCount, chargeCount, amount, validCount, chargeCount, amount)
	_, err = o.Raw(sql).Exec()
	if err != nil {
		o.Rollback()
		return errors.DataBaseError
	}

	// 不扣费
	if chargeCount == 0 {
		// 返回金额
		//sql = "update t_gd_merchants set balance=balance+%f where id=?"
		sql = fmt.Sprintf("update t_gd_merchants set balance=balance+%f where id=?", req.UnitPrice)
		_, err := o.Raw(sql, req.MerchantId).Exec()
		if err != nil {
			o.Rollback()
			return errors.DataBaseError
		}
	}
	o.Commit()

	return nil
}

func ManagementCheckCountCode(ctx context.Context, req *apis.ManagementCheckCountCodeReq, reply *apis.ManagementCheckCountCodeReply) (err error) {
	var merchantDataApiId int64
	defer func() {
		if req.FeeType == 1 {
			go apiWarning(req.AccessCountId)
		}
	}()

	// 如果扣费为金额扣费
	if req.FeeType == 2 {
		return MoneyChargeMode(ctx, req, reply)
	}

	if req.AccessCountId <= 0 {
		return nil
	}

	o := orm.NewOrm()
	sql := "select merchant_data_api_id from t_gd_api_access_count where id=?"
	err = o.Raw(sql, req.AccessCountId).QueryRow(&merchantDataApiId)
	if err != nil {
		setMysqlError(sql, fmt.Sprintf("%s", req.AccessCountId), err)
		return errors.DataBaseError
	}
	//lockKey := fmt.Sprintf("%s-%d", LockCountPrefix, merchantDataApiId)
	//utils.Lock(lockKey)
	//defer utils.UnLock(lockKey)
	if req.Code != "0" {
		err = addAccessFailedCount(o, req.AccessCountId)
	}
	if err != nil {
		return err
	}
	if req.CountType == 0 {
		return nil
	}
	array := strings.Split(req.CountCode, ",")
	for _, v := range array {
		if v == req.Code {
			return nil
		}
	}
	return decreaseAccessCount(o, req.AccessCountId)
}
