// Copyright 2019 utimes.cc. All rights reserved.
// Use of this source code is governed by utimes.cc.

package data_api_check

import (
	"context"
	"gd_auth_check/apis"
	"gd_auth_check/errors"
	"encoding/json"
	"time"

	"github.com/astaxie/beego/orm"
)

func handleSpecialProvider(reply *apis.ManagementCheckProviderApiReply, o orm.Ormer, timeNow string) error {
	m := map[int64]int64{}
	for i, v := range reply.MerchantProviderLimitInfoList {
		if v.DayCountMaxLimit > 0 {
			if _, ok := m[v.ProviderApiId]; ok == false {
				used := int64(0)
				sql := `SELECT
					sum( IF ( t1.count - t2.inventory > 0, t1.count - t2.inventory, 0 ) ) AS total 
					FROM
					t_gd_thirdpart_access_count AS t1
					LEFT JOIN ( SELECT * FROM t_gd_merchant_api_provider_api_relation GROUP BY merchant_child_api_id, provider_api_id ) AS t2 ON t1.provider_api_id = t2.provider_api_id 
					AND t1.merchant_child_api_id = t2.merchant_child_api_id 
					WHERE
					t1.create_time = ? 
					AND t1.provider_api_id = ?`
				err := o.Raw(sql, timeNow, v.ProviderApiId).QueryRow(&used)
				if err != nil && err != orm.ErrNoRows {
					return errors.DataBaseError
				}
				m[v.ProviderApiId] = used
			}
			reply.MerchantProviderLimitInfoList[i].RetainMarginTotalUsed = m[v.ProviderApiId]
		}
	}

	return nil
}

func ManagementCheckProviderApi(ctx context.Context, req *apis.ManagementCheckProviderApiReq, reply *apis.ManagementCheckProviderApiReply) (err error) {
	if req.BaseApiId == 0 && req.MerchantChildApiId == 0 {
		return errors.ArgsError
	}
	var timeLayout = "2006-01-02"
	timeNow := time.Now().Format(timeLayout)
	if req.MerchantChildApiId != 0 {
		sql := `select t1.provider_api_id, t1.group_no, provider_id, enable,state,
			day_count,provider_api_code,thirdpart_host,thirdpart_api_name,
			thirdpart_api_router,thirdpart_api_method,t4.count,t3.count as day_count_max_limit, t3.timeout, t3.filter,
			inventory,retain_margin from
			t_gd_merchant_api_provider_api_relation as t1 left join
			t_gd_merchant_child_data_api as t5 on t1.merchant_child_api_id = t5.id left join 
			t_gd_api_provider_relation  as t2 on t1.provider_api_id = t2.provider_api_id and t1.group_no = t2.group_no
			and t5.api_id = t2.api_id left join t_gd_provider_api as t3 on t2.provider_api_id = t3.id left join
			t_gd_thirdpart_access_count as t4 on t1.merchant_child_api_id =t4.merchant_child_api_id  
			and t4.create_time = ? and t4.provider_api_id = t1.provider_api_id where t1.merchant_child_api_id = ?
			order by t1.group_no, priority`
		o := orm.NewOrm()
		_, err = o.Raw(sql, timeNow, req.MerchantChildApiId).QueryRows(&reply.MerchantProviderLimitInfoList)
		if err != nil && err != orm.ErrNoRows {
			return errors.DataBaseError
		}
		if err = handleSpecialProvider(reply, o, timeNow); err != nil {
			return err
		}
	} else if req.BaseApiId != 0 {
		sql := `select provider_api_id,provider_id, group_no, enable,state,0 as day_count,provider_api_code,thirdpart_host,
			thirdpart_api_name,thirdpart_api_router,thirdpart_api_method,0 as count, t3.count as day_count_max_limit,filter,
			0 as inventory,retain_margin  from t_gd_api as t1
			left join t_gd_api_provider_relation as t2 on t1.id = t2.api_id  
			left join t_gd_provider_api  as t3 on t2.provider_api_id = t3.id 
			left join t_gd_provider as t4 on t3.provider_id = t4.id
			where t1.id = ? order by priority`
		_, err = orm.NewOrm().Raw(sql, req.BaseApiId).QueryRows(&reply.MerchantProviderLimitInfoList)
		if err != nil && err != orm.ErrNoRows {
			return errors.DataBaseError
		}

	}
	for i, v := range reply.MerchantProviderLimitInfoList {
		array := []apis.ProviderParamFilter{}
		json.Unmarshal([]byte(v.Filter), &array)
		reply.MerchantProviderLimitInfoList[i].Filters = array
	}
	return nil
}
