// Copyright 2019 utimes.cc. All rights reserved.
// Use of this source code is governed by utimes.cc.

package data_api_check

import (
	"context"
	"gd_auth_check/apis"
	//	"gd_auth_check/common.in/utils"
	"gd_auth_check/errors"
	"gd_auth_check/rpc_apis"
	"gd_auth_check/rpc_apis/gd_access_log"
	"fmt"
	"strings"
	"time"

	"github.com/astaxie/beego/orm"
)

func providerWarning(mcdaId int64, providerApiId int64) {
	var preq = gd_access_log.WarningProviderCountReq{}
	sql := `SELECT
			a.provider_api_id,
			d.platform_name AS provider_name,
			a.day_count,
			b.thirdpart_api_name AS provider_api_name,
			b.count AS global_day_count,
			b.warning_enable,
			c.count AS day_used,
			( SELECT sum( count ) FROM t_gd_thirdpart_access_count WHERE provider_api_id = a.provider_api_id AND create_time =? ) AS global_day_used 
			FROM
			( t_gd_merchant_api_provider_api_relation AS a LEFT JOIN t_gd_provider_api AS b ON a.provider_api_id = b.id )
			LEFT JOIN t_gd_provider AS d ON d.id = b.provider_id
			LEFT JOIN t_gd_thirdpart_access_count AS c ON c.merchant_child_api_id = a.merchant_child_api_id 
			AND c.provider_api_id = a.provider_api_id 
			AND c.create_time =? 
			WHERE
			a.merchant_child_api_id =? and a.provider_api_id=?`
	nowday := time.Now().Format("2006-01-02")

	o := orm.NewOrm()
	err := o.Raw(sql, nowday, nowday, mcdaId, providerApiId).QueryRow(&preq)
	if err != nil {
		return
	}
	sql = `SELECT
		merchant_name 
		FROM
		t_gd_merchants 
		WHERE
		id = ( SELECT merchant_id FROM t_gd_merchant_data_api WHERE id = ( SELECT merchant_data_api_id FROM t_gd_merchant_child_data_api WHERE id = ? ) )`
	user := ""
	o.Raw(sql, mcdaId).QueryRow(&user)
	if preq.WarningEnable == false {
		return
	}
	preq.MerchantChildApiId = mcdaId
	preq.User = user
	rpc_apis.Crontab.WarningProviderCount(context.Background(), &preq)
}

func getCountInfo(o orm.Ormer, id int64) (countType int, countValues []string) {
	countValue := ""
	o.Raw("select count_type, count_value from t_gd_provider_api where id=?", id).QueryRow(&countType, &countValue)
	if countValue != "" {
		countValues = strings.Split(countValue, ";")
	}
	return
}

func ManagementThirdpartCountIncrease(ctx context.Context, req *apis.ManagementThirdpartCountIncreaseReq, reply *apis.ManagementThirdpartCountIncreaseReply) error {
	//lockKey := fmt.Sprintf("thirdlog-%d-%d", req.MerchantChildApiId, req.ProviderApiId)
	//utils.Lock(lockKey)
	//defer utils.UnLock(lockKey)
	defer func() {
		go providerWarning(req.MerchantChildApiId, req.ProviderApiId)
	}()
	var timeLayout = "2006-01-02"
	timeNow := time.Now().Format(timeLayout)
	o := orm.NewOrm()
	//countType, countValues := getCountInfo(o, req.ProviderApiId)

	sql := fmt.Sprintf("insert into t_gd_thirdpart_access_count(fail_count,"+
		" count, merchant_child_api_id, provider_api_id, create_time) select 0, 0, %d, %d, '%s' "+
		"from dual where not exists (select * from t_gd_thirdpart_access_count where merchant_child_api_id = %d and provider_api_id=%d and create_time='%s')",
		req.MerchantChildApiId, req.ProviderApiId, timeNow, req.MerchantChildApiId, req.ProviderApiId, timeNow)

	if _, err := o.Raw(sql).Exec(); err != nil && strings.Contains(err.Error(), "Duplicate") == false {
		return errors.DataBaseError
	}
	failCount := 0
	if req.State == false {
		failCount = 1
	}
	count := 1
	/*if countType == 1 {
		count = 0
		for _, v := range countValues {
			if v == req.RawCode {
				count = 1
				break
			}
		}
	}*/

	sql = fmt.Sprintf("update t_gd_thirdpart_access_count set "+
		"fail_count= case when create_time <> '%s' then %d else "+
		"fail_count+%d end, count= case when create_time <> '%s' "+
		"then %d else count+%d end, create_time='%s' where provider_api_id=%d and merchant_child_api_id=%d and create_time='%s'", timeNow, failCount, failCount, timeNow, count, count, timeNow, req.ProviderApiId, req.MerchantChildApiId, timeNow)
	o.Raw(sql).Exec()
	return nil
}
