// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package data_api_check

import (
	"encoding/base64"
	"fmt"
	"gd_auth_check/apis"
	"gd_auth_check/common.in/jsonrpc2"
	"gd_auth_check/errors"
	"gd_auth_check/common.in/utils"
	"github.com/astaxie/beego/orm"
	"go.uber.org/zap"
	"golang.org/x/net/context"
)


func decAppPassword(data string) string {
	if data == "" {
		return ""
	}
	bytes, err := base64.StdEncoding.DecodeString(data)
	if err != nil {
		return ""
	}
	ret, _ := utils.AesDecrypt(bytes, []byte(utils.CRYPTO_KEY))
	fmt.Printf("app pass de:%s\n", string(ret))
	return string(ret)
}


func MerchantAppLogin(ctx context.Context, req *apis.MerchantAppLoginReq, reply *apis.MerchantAppLoginReply) error {
	if req.User == "" || req.Password == "" {
		return errors.ArgsError
	}

	p := apis.TGdMerchants{}

	err := orm.NewOrm().Raw("SELECT * FROM t_gd_merchants WHERE app_key = ?", req.User).QueryRow(&p)
	if err != nil {
		if err == orm.ErrNoRows{
			return jsonrpc2.NewJsonError(1003, "用户名错误")
		}
		l.Error("mysql",
			zap.String("sql", "SELECT * FROM t_gd_merchants WHERE app_key = ?"),
			zap.String("args", fmt.Sprintf("app_key:%d", req.User)),
			zap.String("error", err.Error()))
		return errors.DataBaseError
	}

	if decAppPassword(p.AppPassword) != req.Password {
		return errors.CheckMerchantAppPasswordError
	}
	reply.MerchantId = p.Id

	return nil
}
