package gd_access_log

import (
	"context"
	"fmt"
	"time"

	"github.com/smallnest/rpcx/client"
	"github.com/smallnest/rpcx/protocol"
	"github.com/smallnest/rpcx/share"
)

type GdAccessLogXClient struct {
	xcli client.XClient
}

func (a *GdAccessLogXClient) Init(etcdAddrs []string, basePath, servicePath string) {
	opt := client.Option{
		Retries:        3,
		RPCPath:        share.DefaultRPCPath,
		ConnectTimeout: 10 * time.Second,
		SerializeType:  protocol.JSON,
		CompressType:   protocol.None,
		BackupLatency:  10 * time.Millisecond,
	}

	d := client.NewEtcdDiscovery(basePath, servicePath, etcdAddrs, nil)
	a.xcli = client.NewXClient(servicePath, client.Failtry, client.RandomSelect, d, opt)

}

// k8s模式下，服务发现方式需要改为点对点，使用k8s自身的Service进行服务发现，Service名称格式必须是abc-xyz的方式，不能使用"_"
func (a *GdAccessLogXClient) InitForK8s(servicePath string, k8sServiceName string, k8sServicePort string) {
	opt := client.Option{
		Retries:        1,
		RPCPath:        share.DefaultRPCPath,
		ConnectTimeout: 10 * time.Second,
		SerializeType:  protocol.JSON,
		CompressType:   protocol.None,
		BackupLatency:  10 * time.Millisecond,
	}
	sd := client.NewPeer2PeerDiscovery(fmt.Sprintf("tcp@%s:%s", k8sServiceName, k8sServicePort), "")
	a.xcli = client.NewXClient(servicePath, client.Failtry, client.RandomSelect, sd, opt)
}

func (a *GdAccessLogXClient) WarningProviderCount(ctx context.Context, req *WarningProviderCountReq) (reply WarningProviderCountReply, err error) {
	err = a.xcli.Call(ctx, "WarningProviderCount", req, &reply)
	return
}

func (a *GdAccessLogXClient) WarningApiCount(ctx context.Context, req *WarningApiCountReq) (reply WarningApiCountReply, err error) {
	err = a.xcli.Call(ctx, "WarningApiCount", req, &reply)
	return
}
